package token

import error.Severity.CRITICAL
import file.FilePosition
import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.Assertions.assertThrows
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS
import token.TokenFactory.BadCharacterException
import token.TokenFactory.EmptyTokenTextException

@TestInstance(PER_CLASS)
class TokenFactoryTest {

    private lateinit var tokenFactory: TokenFactory
    private lateinit var testPosition: FilePosition

    @BeforeEach
    fun setUp() {
        tokenFactory = TokenFactoryImpl()
        testPosition = FilePosition("testFile", 0, 0)
    }

    private fun createToken(text: String): Token {
        return tokenFactory.createToken(text, testPosition)
    }

    @Test
    fun eofTokenCreation() {
        assertThat(tokenFactory.createEofToken(testPosition)).isInstanceOf(Eof::class.java)
    }

    @Test
    fun leftParenthesisCreation() {
        assertThat(createToken("(")).isInstanceOf(LeftParenthesis::class.java)
    }

    @Test
    fun rightParenthesisCreation() {
        assertThat(createToken(")")).isInstanceOf(RightParenthesis::class.java)
    }

    @Test
    fun quoteMarkCreation() {
        assertThat(createToken("'")).isInstanceOf(QuoteMark::class.java)
    }

    @Test
    fun numberCreation() {
        assertThat(createToken("987")).isInstanceOf(Number::class.java)
    }

    @Test
    fun prefixedNumberCreation() {
        assertThat(createToken("-987")).isInstanceOf(Number::class.java)
    }

    @Test
    fun identifierCreation() {
        assertThat(createToken("identifier")).isInstanceOf(Identifier::class.java)
    }

    @Test
    fun prefixedIdentifierCreation() {
        assertThat(createToken("-identifier")).isInstanceOf(Identifier::class.java)
    }

    @Test
    fun stringCreation() {
        assertThat(createToken("\"string\"")).isInstanceOf(QuotedString::class.java)
    }

    @Test
    fun emptyString_ThrowsException() {
        assertThrows(EmptyTokenTextException::class.java) { createToken("") }
    }

    @Test
    fun emptyTokenTextException_ContainsCorrectAttributes() {
        try {
            createToken("")
        } catch (e: EmptyTokenTextException) {
            assertThat(e.message).isNotEmpty();
            assertThat(e.severity).isEqualTo(CRITICAL)
        }
    }

    @Test
    fun badCharacter_ThrowsException() {
        assertThrows(BadCharacterException::class.java) { createToken("[abc]") }
    }

    @Test
    fun backTickCreation() {
        assertThat(createToken("`")).isInstanceOf(Backquote::class.java)
    }

    @Test
    fun commaCreation() {
        assertThat(createToken(",")).isInstanceOf(Comma::class.java)
    }

    @Test
    fun atSignCreation() {
        assertThat(createToken("@")).isInstanceOf(AtSign::class.java)
    }
}
